use strict;
use Config;
use Test::More;
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand ShellCommand );

RunScript( Sub => sub {
    plan tests => 29;

    my $PerlExe = $Config{osname} eq 'MSWin32' ? 'perl' : $ENV{PERL};

    eval { SystemCommand([$PerlExe, '-E', 'say \'# \', 1+2']); };
    ok( !ExistsError(), 'SystemCommand with parameter array' );

    eval { SystemCommand("$PerlExe -E \"say '# ', 1+2\""); };
    ok( !ExistsError(), 'SystemCommand with command string' );

    eval { SystemCommand([$PerlExe, '-e', 'sa bb cc']) };
    ok( ExistsError(), 'syntax error' );
    cmp_ok( GetError->code, 'eq', 'CommandFailed', 'error code: CommandFailed' );

    eval { SystemCommand(["$PerlExe", '-e', 'exit 5']) };
    ok( ExistsError(), 'exit 5' );
    cmp_ok( GetError->code, 'eq', 'CommandFailed', 'error code: CommandFailed' );
    is( GetError()->vars->{'ExitCode'}, 5, 'exit code' );

    eval { SystemCommand(["$PerlExe", '-e', 'exit 1']) };
    ok( ExistsError(), 'exit 1' );
    cmp_ok( GetError->code, 'eq', 'CommandFailed', 'error code: CommandFailed' );
    is( GetError()->vars->{'ExitCode'}, 1, 'exit code' );

    eval { SystemCommand(["$PerlExe", '-e', 'kill 9, $$']) };
    ok( ExistsError(), 'kill 1' );
    if( $Config{'osname'} eq 'MSWin32' ) {
        cmp_ok( GetError->code, 'eq', 'CommandFailed', 'error code: CommandFailed' );
        is( GetError()->vars->{'ExitCode'}, 9, 'exit code' );
    }
    else {
        cmp_ok( GetError->code, 'eq', 'CommandTerminated', 'error code: CommandTerminated' );
        is( GetError()->vars->{'Signal'}, 9, 'signal' );
    }

    eval { SystemCommand('perlxx') };
    ok( ExistsError(), 'unknown command' );
    cmp_ok( GetError->code, 'eq', 'CommandError', 'error code: CommandError' );
    cmp_ok( GetError->vars->{'ErrorMessage'}, 'eq', 'No such file or directory', 'system error message' );
    cmp_ok( GetError->vars->{'ErrorMessage'}, '==', '2', 'system error code' );
   
    my $Result;
    eval {$Result = SystemCommand('perl -v', {'Capture' => 1}) };
    ok( !ExistsError(), 'capture output: no error' );
    like( $Result, qr/^\nThis is perl/, 'capture output' );

    eval {$Result = SystemCommand('c:\\perlxx.exe', {'Capture' => 1}) };
    ok( ExistsError(), 'capture output: unknown command' );
    cmp_ok( GetError->code, 'eq', 'CommandFailed', 'error code: CommandFailed' );

    # -----------------------------------------------------------------------------
    # Tests for function 'SystemCommand'
    # ----------------------------------------------------------------------------- 
    
    my $TestText = 'Hello, shell command with bash.';
    eval {$Result = ShellCommand('echo ' . $TestText, { 'Capture' => 1 }) };
    ok( !ExistsError(), 'ShellCommand: no error after most simple invocation.' );
    chomp($Result);
    cmp_ok( $Result, 'eq', $TestText, "ShellCommand: most simple invocation.");
    
    eval {ShellCommand( [$PerlExe, '-v'], { 'Print' => 0 }) };
    ok( !ExistsError(), 'ShellCommand: no error after invocation with array.' );
    
    eval {$Result = ShellCommand("$PerlExe -E \"say '# ', 1+2\"", { 'Capture' => 1 }); };
    ok( !ExistsError(), 'ShellCommand: no error after invocation with Perl command.' );
    chomp($Result);
    cmp_ok( $Result, 'eq', '# 3', 'ShellCommand: correct result from Perl command.');
    
    eval { ShellCommand(["$PerlExe", '-e', 'exit 1']) };
    ok( ExistsError(), 'ShellCommand: forward failure from Perl command.' );
    cmp_ok( GetError->code, 'eq', 'CommandFailed', 'ShellCommand: correct error code.' );
    is( GetError()->vars->{'ExitCode'}, 1, 'exit code' );
});
