use strict;
use Test::More tests => 33;
use Test::Exception;
use File::Basename qw( basename );

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Installer::API::Constants qw(
    STOPPED
    PENDING
    RUNNING
    DENIED
    MISSING
);
use DE_EPAGES::Installer::API::Service qw(
    StopService
    StartService
    ExistsService
    StatusService
    StatusUnixService
    EpagesCustomizedMysqlDaemon
    EpagesCustomizedHttpdDaemon
);

my %MAP_STAT = (
    STOPPED() => 'STOPPED',
    PENDING() => 'PENDING',
    RUNNING() => 'RUNNING',
    DENIED()  => 'DENIED ',
    MISSING() => 'MISSING',
    );

RunScript( Sub => sub {

    my $Stat = StatusService('cron');
    ok($Stat->{'cron'} != MISSING, "StatusService('cron') -> $MAP_STAT{$Stat->{'cron'}}");

    foreach my $Service (qw(appserver reqrouter webserver mongodb
                            solr-server solr-server-slave epagesj-server)) {
        my $ServiceExists = ExistsService($Service);
      SKIP: {
          skip("Service not installed: $Service", 1) unless $ServiceExists;
        }
        $Stat = StatusService($Service, {dieIfMissing => 1});
        if ($Stat->{$Service} == STOPPED) {
            StartService($Service);
            $Stat = StatusService($Service, {dieIfMissing => 1});
            ok($Stat->{$Service} == RUNNING, "StartService('$Service')");
            StopService($Service);
            $Stat = StatusService($Service, {dieIfMissing => 1});
            ok($Stat->{$Service} == STOPPED, "StopService('$Service')");
        } else {
            StopService($Service);
            $Stat = StatusService($Service, {dieIfMissing => 1});
            ok($Stat->{$Service} == STOPPED, "StopService('$Service')");
            StartService($Service);
            $Stat = StatusService($Service, {dieIfMissing => 1});
            ok($Stat->{$Service} == RUNNING, "StartService('$Service')");
        }
    }

    my %ServiceCluster = (
        'epages'    => [qw(appserver reqrouter)],
        'epagesj'   => [qw(mongodb solr-server epagesj-server)],
        );
    if (ExistsService('solr-server-slave')) {
        push @{$ServiceCluster{epagesj}}, 'solr-server-slave';
    } else {
      SKIP: {
          skip('Service not installed: solr-server-slave', 2);
        }
    }

    foreach (sort keys %ServiceCluster) {
        my $ServiceExists = ExistsService($_);
      SKIP: {
          skip("Service not installed: $_",
               2 * scalar @{$ServiceCluster{$_}}) unless $ServiceExists;
        }
        my $Origin = StatusService($_, {dieIfMissing => 1});
        StopService($_);
        $Stat = StatusService($_, {dieIfMissing => 1});
        foreach my $Service (@{$ServiceCluster{$_}}) {
            ok($Stat->{$Service} == STOPPED, "StopService('$_') -> $Service");
        }
        StartService($_);
        $Stat = StatusService($_, {dieIfMissing => 1});
        foreach my $Service (@{$ServiceCluster{$_}}) {
            ok($Stat->{$Service} == RUNNING, "StartService('$_') -> $Service");
            StopService($Service) if $Origin->{$Service} == STOPPED;
        }
    }

    foreach (qw(httpd mysqld)) {
        my $RunScript = /httpd/ ? EpagesCustomizedHttpdDaemon()
            : EpagesCustomizedMysqlDaemon;
      SKIP: {
          skip("epages $_ server not installed", 1) unless $RunScript;
        }
        my $Service = basename($RunScript);
        ok($Service, "Valid service name for $RunScript.");
        
        $Stat = StatusUnixService($Service);
        ok($Stat != MISSING, "StatusUnixService('$Service') -> $MAP_STAT{$Stat}");
    }
    
    ##no critic qw(ProtectPrivateSubs)
    my $NonexistentService = DE_EPAGES::Installer::API::Service::_GetDaemon("xxx");
    ok( !($NonexistentService), 'Return false for non-existent service by default.');
    dies_ok {
        DE_EPAGES::Installer::API::Service::_GetDaemon("xxx", {'dieIfMissing' => 1});
    } 'Dies with non-existent service on request.';
    ## use critic
});
